#  MIT License Copyright (c) 2020. Houfu Ang


def get_enforcement(items):
    import spacy
    from spacy.matcher import Matcher
    nlp = spacy.load('en_core_web_sm')
    matcher = Matcher(nlp.vocab)
    financial_penalty_pattern = [{'LOWER': 'financial'},
                                 {'LOWER': 'penalty'},
                                 {'POS': 'ADP'},
                                 {'LOWER': '$'},
                                 {'LIKE_NUM': True}]
    financial_1_id = 'financial_1'
    matcher.add(financial_1_id, [financial_penalty_pattern])
    financial_penalty_pattern2 = [{'LOWER': 'financial'},
                                  {'LOWER': 'penalty'},
                                  {'POS': 'ADP'},
                                  {'LOWER': '$'},
                                  {'LIKE_NUM': True},
                                  {'LOWER': 'and'},
                                  {'LOWER': '$'},
                                  {'LIKE_NUM': True}]
    financial_2_id = 'financial_2'
    matcher.add(financial_2_id, [financial_penalty_pattern2])
    warning_pattern = [{'LOWER': 'warning'},
                       {'POS': 'AUX'},
                       {'LOWER': 'issued'}]
    warning_id = 'warning'
    matcher.add(warning_id, [warning_pattern])
    directions_pattern = [{'LOWER': 'directions'},
                          {'POS': 'AUX'},
                          {'LOWER': 'issued'}]
    directions_id = 'directions'
    matcher.add(directions_id, [directions_pattern])
    print('Adding enforcement information to items.')
    for item in items:
        doc = nlp(item.summary)
        matches = matcher(doc)
        item.enforcement = []
        for match in matches:
            match_id, _, end = match
            if nlp.vocab.strings[financial_2_id] in match:
                span1 = doc[end - 4: end - 3]
                value = ['financial', int(span1.text.replace(',', ''))]
                if not item.enforcement.count(value):
                    item.enforcement.append(value)
                span2 = doc[end - 1:end]
                value = ['financial', int(span2.text.replace(',', ''))]
                if not item.enforcement.count(value):
                    item.enforcement.append(value)
            if nlp.vocab.strings[financial_1_id] in match:
                span = doc[end - 1:end]
                value = ['financial', int(span.text.replace(',', ''))]
                if not item.enforcement.count(value):
                    item.enforcement.append(value)
            if nlp.vocab.strings[warning_id] in match:
                item.enforcement.append(warning_id)
            if nlp.vocab.strings[directions_id] in match:
                item.enforcement.append(directions_id)


def get_decision_citation_all(items):
    print('Adding citation information to items.')
    for item in items:
        get_decision_citation_one(item)


def get_decision_citation_one(item):
    from pdfminer.high_level import extract_text_to_fp
    import requests
    import io
    import re
    r = requests.get(item.download_url)
    item.citation = ''
    item.case_number = ''
    if item.download_url[-3:] == 'pdf':
        with io.BytesIO(r.content) as pdf, io.StringIO() as output_string:
            extract_text_to_fp(pdf, output_string, page_numbers=[0, 1])
            contents = output_string.getvalue()
        match = re.search(r'\[\d{4}]\s+(?:\d\s+)?[A-Z|()]+\s+\[?\d+\]?', contents)
        if match:
            item.citation = match.group()
        match = re.search(r'DP-\w*-\w*', contents)
        if match:
            item.case_number = match.group()


def get_case_references(items):
    print('Adding case reference information to items.')
    import spacy
    from spacy.matcher import Matcher
    from .download_file import get_text_from_pdf
    nlp = spacy.load('en_core_web_sm')
    matcher = Matcher(nlp.vocab)
    citation_pattern = [{"IS_BRACKET": True},
                        {"SHAPE": "dddd"},
                        {"IS_BRACKET": True},
                        {"LIKE_NUM": True, "OP": "?"},
                        {"TEXT": {"REGEX": "^[A-Z]"}, "OP": "?"},
                        {"ORTH": ".", "OP": "?"},
                        {"TEXT": {"REGEX": r"^[A-Z\.]"}},
                        {"ORTH": ".", "OP": "?"},
                        {"LIKE_NUM": True}]
    matcher.add('citations', [citation_pattern])
    # construct referring to index
    for item in items:
        if not hasattr(item, 'citation'):
            get_decision_citation_one(item)
        item.referred_by = []
        item.referring_to = []
        if item.download_url[-3:] == 'pdf':
            doc = nlp(get_text_from_pdf(item))
            matches = matcher(doc)
            for match in matches:
                _, start, end = match
                result_citation = doc[start:end].text
                if (item.referring_to.count(result_citation) == 0) and (result_citation != item.citation):
                    item.referring_to.append(result_citation)
    # constructed referred by index
    for item in items:
        for reference in item.referring_to:
            result_item = next((x for x in items if x.citation == reference), None)
            if result_item:
                if result_item.referred_by.count(item.citation) == 0:
                    result_item.referred_by.append(item.citation)


def scraper_extras(items):
    print('Start adding extra information to items.')
    get_decision_citation_all(items)
    get_enforcement(items)
    get_case_references(items)
    print('End adding extra information to items.')
    return True



#  MIT License Copyright (c) 2020. Houfu Ang

"""
Looks over the PDPC website and creates PDPC Decision objects

Requirements:
* Chrome Webdriver to automate web browser
"""
import re
from dataclasses import dataclass
from datetime import datetime

from selenium.webdriver import Chrome
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.remote.webelement import WebElement


def get_url(item: WebElement):
    link = item.find_element_by_tag_name('a')
    return link.get_property('href')


def get_summary(item: WebElement):
    return item.find_element_by_class_name('rte').text.replace('\n', '. ')


def get_published_date(item: WebElement):
    return datetime.strptime(item.find_element_by_class_name('press__date').text, "%d %b %Y").date()


def get_respondent(item: WebElement):
    link = item.find_element_by_tag_name('a')
    text = link.text
    return re.split(r"\s+[bB]y|[Aa]gainst\s+", text, re.I)[1].strip()


def get_title(item: WebElement):
    return item.find_element_by_tag_name('a').text


class Scraper:
    def __init__(self):
        options = Options()
        # Uncomment the next three lines for a headless chrome
        options.add_argument('--headless')
        options.add_argument('--disable-gpu')
        options.add_argument('--window-size=1920,1080')
        options.add_argument('--no-sandbox')

        self.driver = Chrome(options=options)
        self.driver.implicitly_wait(5)

    def refresh_pages(self):
        group_pages = self.driver.find_element_by_class_name('group__pages')
        return group_pages.find_elements_by_class_name('page-number')

    @classmethod
    def scrape(cls, site_url="https://www.pdpc.gov.sg/Commissions-Decisions/Data-Protection-Enforcement-Cases"):
        print('Starting the scrape')
        self = cls()
        result = []
        try:
            self.driver.get(site_url)
            pages = self.refresh_pages()
            for page_count in range(len(pages)):
                pages[page_count].click()
                print("Now at Page ", page_count)
                pages = self.refresh_pages()
                decisions = self.driver.find_elements_by_class_name('press-item')
                for decision in decisions:
                    item = PDPCDecisionItem.from_element(decision)
                    print("Added:", item)
                    result.append(item)
        finally:
            self.driver.close()
            print('Ending scrape.')
        return result


@dataclass
class PDPCDecisionItem:
    published_date: datetime.date
    respondent: str
    title: str
    summary: str
    download_url: str

    @classmethod
    def from_element(cls, decision: WebElement):
        published_date = get_published_date(decision)
        respondent = get_respondent(decision)
        title = get_title(decision)
        summary = get_summary(decision)
        download_url = get_url(decision)
        return cls(published_date, respondent, title, summary, download_url)

    def __str__(self):
        return "PDPCDecision object: {} {}".format(self.published_date, self.respondent)
